/*
 * @(#)ActionGroup.java
 *
 * Copyright (c) 2002, Jang-Ho Hwang
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 	1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 	2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 	3. Neither the name of the Jang-Ho Hwang nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *    $Id: ActionGroup.java,v 1.5 2002/03/15 10:25:09 xrath Exp $
 */
package rath.jmsn;

import java.io.IOException;
import java.awt.event.ActionEvent;
import javax.swing.JOptionPane;

import rath.msnm.UserStatus;
import rath.msnm.MSNMessenger;
import rath.msnm.entity.MsnFriend;
import rath.jmsn.ui.LoginDialog;
import rath.jmsn.ui.AddFriendDialog;
import rath.jmsn.ui.AddConfirmDialog;
import rath.jmsn.ui.AboutDialog;
import rath.jmsn.ui.OptionDialog;
import rath.jmsn.ui.ReportDialog;
import rath.jmsn.util.GlobalProp;
import rath.jmsn.util.UserStatusBox;
import rath.jmsn.util.Msg;
/**
 * Ǵ ActionEvent ϴ Ŭ̴.
 *
 * @author Jang-Ho Hwang, rath@linuxkorea.co.kr
 * @version $Id: ActionGroup.java,v 1.5 2002/03/15 10:25:09 xrath Exp $
 */
public class ActionGroup implements UserStatus
{
	private MainFrame main = null;
	private MSNMessenger msnm = null;
	private BuddyTree buddies = null;

	private ReportDialog dialogBugReport = null;
	private ReportDialog dialogFeatureReq = null;

	public ActionGroup( MainFrame main, BuddyTree buddies )
	{
		this.main = main;
		this.buddies = buddies;
		this.msnm = main.getMessenger();
	}

	public boolean actionLogin()
	{
		LoginDialog dialog = new LoginDialog(main);
		dialog.show();

		if( dialog.isConfirm() )
		{
			String login = dialog.getLoginName();
			String pass = dialog.getPassword();
		    processLogin( login, pass, dialog.getInitialStatus() );
			if( dialog.isRememberPassword() )
				login = login + ":" + pass;
			MainFrame.getGlobalProp().set( "last.login", login );
			return true;
		}
		return false;
	}

	public void actionLogout()
	{
		main.logout();
	}

	private void processLogin( String login, String pass, String status )
	{
		msnm.setInitialStatus( status );
		msnm.login( login, pass );
		main.showLogging();
	}

	/**
	 *   , Ⱥ toggle
	 */
	public void actionToggleOffView()
	{
		buddies.toggleOfflineView();
	}

	/**
	 * ο ģ ϱ  ̾α׸ .
	 */
	public void actionAddFriend()
	{
		AddFriendDialog add = new AddFriendDialog( main, msnm );
		add.show();
	}

	/**
	 *  BuddyTree õ ڰ ִ ˻ϰ  ִٸ,
	 *   , OptionPane ѹ Ŀ,   Ѵ.
	 */
	public void actionRemoveFriend()
	{
		MsnFriend friend = buddies.getSelectedFriend();
		if( friend!=null )
		{
			String loginName = friend.getLoginName();
			if( JOptionPane.showConfirmDialog( main,
				Msg.get("remdlg.query.remove.content", loginName),
				Msg.get("remdlg.query.remove.title"),
				JOptionPane.YES_NO_OPTION )==JOptionPane.YES_OPTION )
			{
				try
				{
					msnm.removeFriend( loginName );
				}
				catch( IOException e ) {}
			}
		}
	}

	public void actionStatusChange( ActionEvent evt )
	{
		String info = evt.getActionCommand();
		String mode = UserStatusBox.getStatusAtFormattedValue(info);
		if( mode==null ) return;
		try
		{
			main.setMyStatus( mode );
			msnm.setMyStatus( mode );
		}
		catch( IOException e ) { e.printStackTrace(); }
	}

	public void actionBuddyView( ActionEvent e )
	{
		String cmd = e.getActionCommand();

		if( cmd.equals(Msg.get("menu.item.friendview.fn")) )
		{
			buddies.setBuddyView( BuddyRenderer.VIEW_FRIENDLY_NAME );
		}
		else
		if( cmd.equals(Msg.get("menu.item.friendview.ln")) )
		{
			buddies.setBuddyView( BuddyRenderer.VIEW_LOGIN_NAME );
		}
		if( cmd.equals(Msg.get("menu.item.friendview.fnln")) )
		{
			buddies.setBuddyView(
				BuddyRenderer.VIEW_LOGIN_NAME | BuddyRenderer.VIEW_FRIENDLY_NAME );
		}
	}

	public void actionExit()
	{
		System.exit(0);
	}

	public void actionAbout()
	{
		new AboutDialog(main).show();
	}

	public void actionOption()
	{
		new OptionDialog(main, msnm).show();
	}

	public void actionBugReport()
	{
		new ReportDialog(main, msnm, Msg.get("title.bugreport"),
			ReportDialog.REPORT_BUG).show();
	}

	public void actionFeatureRequest()
	{
		new ReportDialog(main, msnm, Msg.get("title.reqfeature"),
			ReportDialog.REPORT_FEATURE).show();
	}

	public void actionRefresh()
	{
		main.buddies.syncAllUsers();
	}
}
