/*
 * @(#)ChatDialog.java
 *
 * Copyright (c) 2002, Jang-Ho Hwang
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 	1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 	2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 	3. Neither the name of the Jang-Ho Hwang nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *    $Id: ChatDialog.java,v 1.3 2002/03/16 19:31:39 xrath Exp $
 */
package rath.jmsn.ui;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.Vector;
import java.util.Hashtable;
import javax.swing.*;
import javax.swing.text.*;

import rath.msnm.UserStatus;
import rath.msnm.LocalCopy;
import rath.msnm.MSNMessenger;
import rath.msnm.SwitchboardSession;
import rath.msnm.event.MsnAdapter;
import rath.msnm.entity.MsnFriend;
import rath.msnm.msg.MimeMessage;
import rath.msnm.ftp.VolatileDownloader;
import rath.msnm.ftp.VolatileTransferServer;

import rath.jmsn.ToolBox;
import rath.jmsn.MainFrame;
import rath.jmsn.entity.FileItem;
import rath.jmsn.file.FileProgress;
import rath.jmsn.file.ChatLogWriter;
import rath.jmsn.util.MusicBox;
import rath.jmsn.util.Msg;
/**
 * ȭҶ Ǵ ̾α̴.
 *
 * @author Jang-Ho Hwang, rath@linuxkorea.co.kr
 * @version $Id: ChatDialog.java,v 1.3 2002/03/16 19:31:39 xrath Exp $
 */
public class ChatDialog extends JFrame implements ToolBox, DialogAppender
{
	private static final SimpleDateFormat sdf = new SimpleDateFormat("mm:ss");

	private MSNMessenger msnm = null;
	private MsnFriend me = null;
	private JScrollPane chatPane = null;
	private JScrollPane inputPane = null;
	private JButton filesendButton = null;
	private ChatArea chatArea = null;
	private JTextArea inputArea = null;
	private JLabel statusLabel = null;
	private SwitchboardSession session = null;
	private boolean isSessionClose = false;
	private WindowListener wl = null;

	private JPanel topPanel = null;

	private MsnFriend you = null;
	private ArrayList queue = new ArrayList();
	private ChatLogWriter log = null;

	private Vector fileQueue = new Vector();
	private Hashtable fileCookieMap = new Hashtable();
	private MsnAdapter fileAdapter = new FileAdapter();

	private long lastMessagingTime = 0L;

	public ChatDialog( Frame owner, MSNMessenger msnm, String targetName, MsnFriend i )
	{
		setTitle( targetName );

		this.me = i;
		this.msnm = msnm;

		init();
	}

	/**
	 * κ â  Ǵ constructor̴.
	 */
	public ChatDialog( Frame owner, MSNMessenger msnm, SwitchboardSession session,
		MsnFriend i )
	{
		setTitle( session.getMsnFriend().getLoginName() );

		this.me = i;
		this.you = session.getMsnFriend();
		this.msnm = msnm;
		this.session = session;

		MusicBox.play( MusicBox.SOUND_MESSAGE_2 );
		prepareLogWriter( you.getLoginName() );
		init();
	}

	private void init()
	{
		createComponents();
		createKeyMap();

		this.addComponentListener( new DialogStatus() );
		this.addFocusListener( new FocusListener() {
			public void focusGained( FocusEvent e )
			{
				inputArea.requestFocus();
			}
			public void focusLost( FocusEvent e )
			{
				chatArea.requestFocus();
			}
		});
		this.msnm.addMsnListener( fileAdapter );

		Dimension size = Toolkit.getDefaultToolkit().getScreenSize();
		int x = (int)(Math.random() * (double)(size.height-getSize().width));
		int y = (int)(Math.random() * (double)(size.height-getSize().height));

		setLocation( x, y );
		setDefaultCloseOperation( DISPOSE_ON_CLOSE );
	}

	private void prepareLogWriter( String targetName )
	{
		if( this.log!=null )
			return;

		String logdir = msnm.getLocalCopy().getProperty( MainFrame.CHATLOG_PROP );
		File file = new File( logdir, targetName + ".txt" );
		try {
			this.log = new ChatLogWriter( file );
		} catch( IOException e ) { System.err.println( "can't write log file." ); }
	}

	private class FileAdapter extends MsnAdapter
	{
		public void fileReceiveStarted( VolatileDownloader down )
		{
			String cookie = down.getCookie();
			FileProgress prog = null;
			if( (prog=(FileProgress)fileCookieMap.remove(cookie))!=null )
			{
				prog.setTransfer( down );
			}
		}

		public void fileSendStarted( VolatileTransferServer vts )
		{
			String cookie = vts.getCookie();

			FileProgress prog = null;
			if( (prog=(FileProgress)fileCookieMap.remove(cookie))!=null )
			{
				prog.setTransfer( vts );
			}
		}

		public void fileSendError( VolatileTransferServer server, Throwable e )
		{
			e.printStackTrace();
		}

		public void fileSendAccepted( SwitchboardSession ss, int cookie )
		{
			if( session==null ) return;

			if( ss.getSessionId().equals( session.getSessionId() ) )
			{
				final FileProgress prog = new FileProgress( ChatDialog.this );
				fileCookieMap.put( String.valueOf(cookie), prog );

				SwingUtilities.invokeLater( new Runnable() {
					public void run()
					{
						topPanel.add( prog );
						topPanel.doLayout();
						topPanel.validate();
					}
				});

				MimeMessage msg = new MimeMessage();
				msg.setMessage( Msg.get("chatdlg.filesend.accept", you.getLoginName()) );
				appendMessage( msg );
			}
		}

		public void fileSendRejected( SwitchboardSession ss, int cookie, String reason )
		{
			if( session==null ) return;

			if( ss.getSessionId().equals( session.getSessionId() ) )
			{
				MimeMessage msg = new MimeMessage();
				msg.setMessage( Msg.get("chatdlg.filesend.reject", you.getLoginName()) );
				appendMessage( msg );
			}
		}
	}

	public void addFileItem( final FileItem item )
	{
		fileQueue.add(item);

		MimeMessage msg = new MimeMessage();
		msg.setMessage( Msg.get("chatdlg.filerecv.notify",
			you.getFormattedFriendlyName(), item.getName()+"("+item.getKBSize()+")") );
		appendMessage( msg );
	}

	private void createKeyMap()
	{
		JPanel panel = (JPanel)getContentPane();
		ComponentInputMap im = new ComponentInputMap(panel);
		im.put( KeyStroke.getKeyStroke(KeyEvent.VK_T, InputEvent.ALT_MASK), "FileAccept" );
		im.put( KeyStroke.getKeyStroke(KeyEvent.VK_D, InputEvent.ALT_MASK), "FileReject" );

		ActionMap am = panel.getActionMap();
		am.put( "FileAccept", new AbstractAction() {
			public void actionPerformed( ActionEvent e )
			{
				if( fileQueue.size()==0 )
					return;

				FileItem item = (FileItem)fileQueue.remove(0);
				SwitchboardSession ss = item.getSession();
				LocalCopy local = msnm.getLocalCopy();
				FileProgress prog = new FileProgress(ChatDialog.this);
				try
				{
					File file = new File(
						local.getProperty(MainFrame.DOWNLOAD_PROP),
						item.getName() );
					fileCookieMap.put( String.valueOf(item.getCookie()), prog );
					ss.acceptFileReceive( item.getCookie(), file );

				}
				catch( IOException ex ) { ex.printStackTrace(); }

				topPanel.add( prog );
				topPanel.doLayout();
				topPanel.validate();
			}
		});
		am.put( "FileReject", new AbstractAction() {
			public void actionPerformed( ActionEvent e )
			{
				if( fileQueue.size()==0 )
					return;

				FileItem item = (FileItem)fileQueue.remove(0);
				SwitchboardSession ss = item.getSession();
				try {
					ss.rejectFileReceive( item.getCookie() );
				} catch( IOException ex ) { ex.printStackTrace(); }

				MimeMessage msg = new MimeMessage();
				msg.setMessage( Msg.get("chatdlg.filerecv.reject") );
				appendMessage( msg );
			}
		});
		panel.setInputMap( JComponent.WHEN_IN_FOCUSED_WINDOW, im );

		addKeyListener( new KeyAdapter() {
			public void keyPressed( KeyEvent e )
			{
				if( e.getKeyCode()==KeyEvent.VK_ESCAPE )
					dispose();
			}
		});
	}

	private class DialogStatus extends ComponentAdapter
	{
		public void componentShown( ComponentEvent e )
		{
			inputArea.requestFocus();
		}
	}

	public void sessionClosed()
	{
		this.session = null;
		isSessionClose = true;
	}

	public void userOffline()
	{
		MimeMessage msg = new MimeMessage();
		msg.setMessage( Msg.get("chatdlg.msg.offline", you.getLoginName()) );
		appendMessage( msg );

		filesendButton.setEnabled( false );
		inputArea.setEnabled( false );
		chatArea.requestFocus();
	}

	private void createComponents()
	{
		setSize( 350, 350 );

		JPanel panel = (JPanel)getContentPane();

		chatArea = new ChatArea() {
			public void acceptFiles( List list )
			{
				for(Iterator i=list.iterator(); i.hasNext(); )
					sendFileImpl( (File)i.next() );
			}
		    public void inviteFriend( String loginName )
			{
				if( session!=null )
				{
					try
					{
						session.inviteFriend( loginName );

						MimeMessage msg = new MimeMessage();
						msg.setMessage( Msg.get("chatdlg.msg.invite", loginName) );
						appendMessage( msg );
					}
					catch( IOException e ) {}
				}
			}
		};
		chatArea.setFont( FONT_10 );
		chatArea.setLineWrap( true );
		chatArea.setEditable( false );
		chatArea.append( Msg.get("chatdlg.msg.top") );

		inputArea = new JTextArea();
		inputArea.setFont( FONT_10 );
		inputArea.setLineWrap( true );

		statusLabel = new JLabel("");
		statusLabel.setFont( FONT_10 );
		statusLabel.setPreferredSize( new Dimension(100, 24) );

		chatPane = new JScrollPane( chatArea );
		inputPane = new JScrollPane( inputArea );
		inputPane.setPreferredSize( new Dimension(100, 50) );

		JPanel bottomPanel = new JPanel(new BorderLayout());
		bottomPanel.add( inputPane, "Center" );
		bottomPanel.add( statusLabel, "South" );

		topPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 4, 4));
		topPanel.setPreferredSize( new Dimension(100,38) );
		topPanel.setBorder( BorderFactory.createEtchedBorder() );

		filesendButton = new JButton(Msg.get("button.file.send"));
		filesendButton.setFont( FONT );
		filesendButton.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e )
			{
				prepareSendFile();
			}
		});
		topPanel.add( filesendButton );

		panel.add( topPanel, "North" );
		panel.add( chatPane, "Center" );
		panel.add( bottomPanel, "South" );

		InputMap im = inputArea.getInputMap();
		ActionMap am = inputArea.getActionMap();

		AbstractAction newlineAction = new AbstractAction()
		{
			public void actionPerformed( ActionEvent e )
			{
				inputArea.append("\n");
			}
		};

		im.put( KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, InputEvent.CTRL_MASK), "newline" );
		im.put( KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, InputEvent.SHIFT_MASK), "newline" );
		im.put( KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), "send" );
		im.put( KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "dispose" );

		am.put( "newline", newlineAction );
		am.put( "send", new AbstractAction() {
			public void actionPerformed( ActionEvent e )
			{
				sendMessage();
			}
		});
		am.put( "dispose", new AbstractAction() {
			public void actionPerformed( ActionEvent e )
			{
				dispose();
			}
		});
	}

	public void appendMessage( MimeMessage msg )
	{
		Appender appender = new Appender();
		appender.setAlert( true );
		appender.setMessage( msg );

		if( log!=null )
			log.println( msg.getMessage() );

		if( !SwingUtilities.isEventDispatchThread() )
			SwingUtilities.invokeLater( appender );
		else
			appender.run();
	}

	public void appendMessage( MsnFriend friend, MimeMessage msg )
	{
		Appender appender = new Appender();
		appender.setAlert( false );
		appender.setFriend( friend );
		appender.setMessage( msg );

		if( friend!=me )
			this.lastMessagingTime = System.currentTimeMillis();

		if( log!=null )
			log.println( friend, msg.getMessage() );

		if( !SwingUtilities.isEventDispatchThread() )
			SwingUtilities.invokeLater( appender );
		else
			appender.run();
	}

	protected void prepareSendFile()
	{
		if( isSessionClose )
		{
			isSessionClose = false;
			try {
				msnm.doCall( you.getLoginName() );
			} catch( IOException e ) {}
		}

		String downdir = msnm.getLocalCopy().getProperty(MainFrame.DOWNLOAD_PROP);
		JFileChooser file = new JFileChooser(downdir);
		file.setMultiSelectionEnabled( true );
		file.setFileSelectionMode( JFileChooser.FILES_AND_DIRECTORIES );
		if( file.showOpenDialog( this )==JFileChooser.APPROVE_OPTION )
		{
			File[] fs = file.getSelectedFiles();
		    for(int i=0; i<fs.length; i++)
			{
				sendFileImpl( fs[i] );
			}
		}
	}

	/**
	 * ش  Ѵ,  û  丮   
	 * ѹ ڿ  , 丮.zip  Ͽ Ѵ.
	 * <p>
	 * ش  ý temp directory Ǹ,   ǰų,
	 *  ߴܵǸ, Ǿ Ѵ.
	 *
	 * @param f
	 */
	protected void sendFileImpl( File f )
	{
		if( f.isDirectory() )
		{
			if( JOptionPane.showConfirmDialog( this, Msg.get("chatdlg.query.dirsend.content", f.getName()),
				Msg.get("chatdlg.query.dirsend.title"), JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE )
				==JOptionPane.NO_OPTION )
				return;
			ZippedDirectoryDialog zd = new ZippedDirectoryDialog(this, f);
			zd.show();
			f = zd.getZippedFile();
			f.deleteOnExit();
		}

		MimeMessage msg = new MimeMessage();
		if( session!=null )
		{
			try {
				msnm.sendFileRequest( you.getLoginName(), f, session );
				msg.setMessage( Msg.get("chatdlg.filesend.request.ok", you.getLoginName(), f.getName()) );
			}
			catch( IOException e )
			{
				msg.setMessage( Msg.get("chatdlg.filesend.request.fail") );
			}
			appendMessage( msg );
		}
		else
		{
			msg.setMessage( Msg.get("chatdlg.filesend.request.fail") );
			appendMessage( msg );
		}
	}

	/**
	 * ܼ ޽ â  ߰ϴ  Ѵ.
	 */
	private class Appender implements Runnable
	{
		boolean alert = false;
		MsnFriend friend;
		MimeMessage msg;
		JViewport view;
		Rectangle rect = new Rectangle();

		public Appender()
		{
			view = chatPane.getViewport();
			rect.x = 0;
			rect.width = 1;
			rect.height = 300;
		}

		public void setAlert( boolean isAlert )
		{
			this.alert = isAlert;
		}

		public void setFriend( MsnFriend friend )
		{
			this.friend = friend;
		}

		public void setMessage( MimeMessage msg )
		{
			this.msg = msg;
		}

		public void run()
		{
			if( alert )
			{
				chatArea.append( msg.getMessage() + "\n" );
				if( getFocusOwner()==null )
					MusicBox.play( MusicBox.SOUND_MESSAGE_1 );
			}
			else
			{
				chatArea.append( Msg.get("chatdlg.chat", friend.getFormattedFriendlyName()) );
				chatArea.append( msg.getMessage() + "\n" );
				if( getFocusOwner()==null && friend!=me )
					MusicBox.play( MusicBox.SOUND_MESSAGE_2 );
			}

			rect.y = chatArea.getSize().height;
			view.scrollRectToVisible( rect );
			statusLabel.setText( "" );
		}
	}

	protected void sendMessage()
	{
		String input = inputArea.getText();
		if( input.trim().length()==0 )
			return;
		inputArea.setText( "" );

		if( me.getStatus().equals(UserStatus.INVISIBLE) )
		{
			inputArea.setText( Msg.get("chatdlg.msg.notsend.offline") );
			filesendButton.setEnabled( false );
			inputArea.setEnabled( false );
			return;
		}

		MimeMessage msg = new MimeMessage(input);
		msg.setKind( MimeMessage.KIND_MESSAGE );

		appendMessage( this.me, msg );

		try
		{
			if( session!=null )
				session.sendInstantMessage( msg );
			else
			{
				queue.add( msg );
				if( isSessionClose )
				{
					isSessionClose = false;
					msnm.doCall( you.getLoginName() );
				}
			}
		}
		catch( java.io.IOException e )
		{
			e.printStackTrace();
		}
	}

	public void addWindowListener( WindowListener wl )
	{
		super.addWindowListener( wl );
		this.wl = wl;
	}

	public void dispose()
	{
		if( topPanel.getComponentCount()>1 )
		{
			JOptionPane.showMessageDialog(this,
				Msg.get("chatdlg.msg.transfer.4eva"),
				Msg.get("title.alarm"), JOptionPane.INFORMATION_MESSAGE );
		}

		if( log!=null )
			log.close();

		wl.windowClosing(
			new WindowEvent(this, WindowEvent.WINDOW_CLOSING) );
		msnm.removeMsnListener( fileAdapter );
		super.dispose();
	}

	public void close()
	{
		try
		{
			if( session!=null )
			{
				session.close();
				session = null;
			}
		}
		catch( Exception e )
		{
			System.err.println( e );
		}
	}

	public void setSession( SwitchboardSession session )
	{
		if( this.session!=null )
		{
			if( !this.session.getSessionId().equals(session.getSessionId()) )
			{
				try
				{
					this.session.close();
				}
				catch( IOException e ) {}
			}
		}
		this.session = session;
		this.you = session.getMsnFriend();
		this.lastMessagingTime = System.currentTimeMillis();

		prepareLogWriter( you.getLoginName() );

		MimeMessage mm = new MimeMessage();
		mm.setMessage( Msg.get("chatdlg.chat.opened") );
		appendMessage( mm );

		synchronized( queue )
		{
			if( queue.size()!=0 )
			{
				for(Iterator i=queue.iterator(); i.hasNext(); )
				{
					MimeMessage msg = (MimeMessage)i.next();
					try {
						session.sendInstantMessage( msg );
					}
					catch( IOException e )
					{
						msg.setMessage( Msg.get("chatdlg.chat.error", msg.getMessage()) );
						appendMessage( msg );
					}
					finally
					{
						i.remove();
					}
				}
			}
		}
	}

	public SwitchboardSession getSession()
	{
		return this.session;
	}

	/**
	 *  ȭ  ۵ ð ȯѴ. CPU Time̴.
	 */
	public long getLastMessagingTime()
	{
		return this.lastMessagingTime;
	}

	public void whoTyping( final String userview )
	{
		SwingUtilities.invokeLater( new Runnable() {
			public void run()
			{
				statusLabel.setText( Msg.get("chatdlg.msg.typing", userview, sdf.format(new Date())) );
			}
		});
	}
}
