/*
 * @(#)OptionDialog.java
 *
 * Copyright (c) 2002, Jang-Ho Hwang
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 	1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 	2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 	3. Neither the name of the Jang-Ho Hwang nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *    $Id: OptionDialog.java,v 1.2 2002/03/13 09:40:39 xrath Exp $
 */
package rath.jmsn.ui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import java.io.File;
import java.io.IOException;

import rath.msnm.MSNMessenger;
import rath.msnm.LocalCopy;
import rath.msnm.entity.MsnFriend;

import rath.jmsn.ToolBox;
import rath.jmsn.MainFrame;
import rath.jmsn.util.MusicBox;
import rath.jmsn.util.Msg;
/**
 * Option dialog.
 *
 * @author Jang-Ho Hwang, rath@linuxkorea.co.kr
 * @version $Id: OptionDialog.java,v 1.2 2002/03/13 09:40:39 xrath Exp $
 */
public class OptionDialog extends DefaultDialog implements ToolBox
{
	private JTextField nickField = null;
	private JTextField downField = null;
	private JTextField chatlogField = null;

	private JButton findButton0, findButton1;
	private JCheckBox loginCheck = null;
	private JCheckBox messageCheck = null;

	private MSNMessenger msnm = null;
	private String initFriendlyName = null;

	public OptionDialog( Frame owner, MSNMessenger msnm )
	{
		super( owner );

		this.msnm = msnm;

		setTitle( Msg.get("title.option") );
		createUI();
	}

	private void createUI()
	{
		setSize( 400, 360 );

		JPanel panel = (JPanel)getContentPane();

		JTabbedPane pane = new JTabbedPane();
		pane.setBorder( BorderFactory.createEmptyBorder(5,5,5,5) );

		pane.add( createGeneralTab(), "General" );

		panel.add( pane, "Center" );
		panel.add( createBottomPanel(), "South" );
	}

	private JPanel createGeneralTab()
	{
		JPanel panel = new JPanel();
		BoxLayout layout = new BoxLayout(panel, BoxLayout.Y_AXIS);
		panel.setBorder( BorderFactory.createEmptyBorder(10,10,10,10) );
		panel.setLayout( layout );

		JPanel nickPanel = new JPanel(new BorderLayout(5,5));
		nickPanel.setMaximumSize( new Dimension(Short.MAX_VALUE, 24) );
		nickPanel.setAlignmentX( 0.0f );
		JLabel nickLabel = new JLabel( Msg.get("label.friendlyname") );
		nickLabel.setFont( FONT );
		nickField = new JTextField();
		nickField.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e )
			{
				processConfirm();
			}
		});
		nickField.setFont( FONT );
		nickPanel.add( nickLabel, "West" );
		nickPanel.add( nickField, "Center" );
		panel.add( nickPanel );

		panel.add( createDummyPanel(5) );

		loginCheck = new JCheckBox(Msg.get("label.sound.login"));
		loginCheck.setFont( FONT );
		messageCheck = new JCheckBox(Msg.get("label.sound.msg"));
		messageCheck.setFont( FONT );

		panel.add( loginCheck );
		panel.add( messageCheck );

		panel.add( createDummyPanel(30) );

		JLabel downLabel = new JLabel( Msg.get("label.folder.download") );
		downLabel.setFont( FONT );
		panel.add( downLabel );

		JPanel downPanel = new JPanel(new BorderLayout(5,5));
		downPanel.setAlignmentX( 0.0f );
		downPanel.setMaximumSize( new Dimension(Short.MAX_VALUE,24) );
		downField = new JTextField();
		downField.setEditable( false );
		downField.setFont( FONT );
		findButton0 = new JButton( Msg.get("button.find") );
		findButton0.setFont( FONT );
		findButton0.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e )
			{
				File dir = selectDirectory( downField.getText() );
				if( dir!=null )
					downField.setText( dir.getAbsolutePath() );
			}
		});
		downPanel.add( downField, "Center" );
		downPanel.add( findButton0, "East" );
		panel.add( downPanel );

		panel.add( createDummyPanel(5) );

		JLabel logLabel = new JLabel( Msg.get("label.folder.chatlog") );
		logLabel.setFont( FONT );
		panel.add( logLabel );

		JPanel logPanel = new JPanel(new BorderLayout(5,5));
		logPanel.setAlignmentX( 0.0f );
		logPanel.setMaximumSize( new Dimension(Short.MAX_VALUE,24) );
		chatlogField = new JTextField();
		chatlogField.setEditable( false );
		chatlogField.setFont( FONT );
		findButton1 = new JButton( Msg.get("button.find") );
		findButton1.setFont( FONT );
		findButton1.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e )
			{
				File dir = selectDirectory( chatlogField.getText() );
				if( dir!=null )
					chatlogField.setText( dir.getAbsolutePath() );
			}
		});
		logPanel.add( chatlogField, "Center" );
		logPanel.add( findButton1, "East" );
		panel.add( logPanel );

		return panel;
	}

	private JPanel createDummyPanel( int height )
	{
		JPanel d0 = new JPanel();
		d0.setAlignmentX( 0.0f );
		d0.setMaximumSize( new Dimension(Short.MAX_VALUE, height) );
		return d0;
	}

	private JPanel createBottomPanel()
	{
		JPanel bottomPanel = new JPanel(new FlowLayout(FlowLayout.CENTER, 10,4));
		JButton confirmButton = new JButton(Msg.get("button.ok"));
		confirmButton.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e )
			{
				processConfirm();
			}
		});
		confirmButton.setFont( FONT );
		JButton cancelButton = new JButton(Msg.get("button.cancel"));
		cancelButton.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e )
			{
				dispose();
			}
		});
		cancelButton.setFont( FONT );
		bottomPanel.add( confirmButton );
		bottomPanel.add( cancelButton );
		return bottomPanel;
	}

	private void processConfirm()
	{
		if( !msnm.isLoggedIn() )
		{
			dispose();
			return;
		}

		File downDir = new File( downField.getText() );
		downDir.mkdirs();
		File logDir = new File( chatlogField.getText() );
		logDir.mkdirs();

		boolean sndLogin = loginCheck.isSelected();
		boolean sndMessage = messageCheck.isSelected();

		LocalCopy local = msnm.getLocalCopy();
		local.setProperty( MainFrame.DOWNLOAD_PROP, downDir.getAbsolutePath() );
		local.setProperty( MainFrame.CHATLOG_PROP, logDir.getAbsolutePath() );
		local.setProperty( MainFrame.SOUND_LOGIN_PROP+".enable", String.valueOf(sndLogin) );
		local.setProperty( MainFrame.SOUND_MESSAGE_PROP+".enable", String.valueOf(sndMessage) );
		local.storeInformation();

		MusicBox.setEnabled( MusicBox.SOUND_LOGIN, sndLogin );
		MusicBox.setEnabled( MusicBox.SOUND_MESSAGE_1, sndMessage );
		MusicBox.setEnabled( MusicBox.SOUND_MESSAGE_2, sndMessage );

		try
		{
			if( msnm!=null && msnm.isLoggedIn() &&
				!initFriendlyName.equals(nickField.getText()) )
				msnm.setMyFriendlyName( nickField.getText() );
		}
		catch( IOException e ) {}

		dispose();
	}

	private File selectDirectory( String current )
	{
		JFileChooser file = new JFileChooser(current);
		file.setFileSelectionMode( JFileChooser.DIRECTORIES_ONLY );
		if( file.showOpenDialog(this)==JFileChooser.APPROVE_OPTION )
		{
			return file.getSelectedFile();
		}
		return null;
	}

	/**
	 * غ۾  Ѵ.
	 */
	private void processReady()
	{
		if( msnm.isLoggedIn() )
		{
			this.initFriendlyName = msnm.getOwner().getFormattedFriendlyName();
			nickField.setText( initFriendlyName );
		}
		else
		{
			nickField.setText( Msg.get("label.login.please") );
			nickField.setEnabled( false );

			loginCheck.setEnabled( false );
			messageCheck.setEnabled( false );

			findButton0.setEnabled( false );
			findButton1.setEnabled( false );
			return;
		}

		LocalCopy local = msnm.getLocalCopy();

		boolean loginEnable = local.getPropertyBoolean(
			MainFrame.SOUND_LOGIN_PROP + ".enable", true);
		boolean messageEnable = local.getPropertyBoolean(
			MainFrame.SOUND_MESSAGE_PROP + ".enable", true);

		loginCheck.setSelected( loginEnable );
		messageCheck.setSelected( messageEnable );

		String downdir = local.getProperty( MainFrame.DOWNLOAD_PROP );
		String logdir = local.getProperty( MainFrame.CHATLOG_PROP );

		new File(downdir).mkdirs();
		new File(logdir).mkdirs();

		downField.setText( downdir );
		chatlogField.setText( logdir );
	}

	public void show()
	{
		processReady();
		super.show();
	}
}
